#delimit;

/* Designed to be run on a dataset created with Patrick Royston's ice
*  program.  misumm computes summary statistics for each of the
*  imputation subsets and returns all the results in the same form as
*  summ2. 
*
*  Weights are supported.
*
*  Version 1.0, Doug McKee, dmckee@ucla.edu, 20jun2008
*/

capture program drop misumm;
program misumm ,rclass;
	syntax varlist(min=1) [if] [in] [fweight aweight iweight];

	di;
	di in text "    Variable {c |}      Obs"
	      	   _col(33) "Mean    "
		   _col(41) "Std. Dev.";
	di in text "{hline 13}{c +}{hline 34}";

	return local varlist `"`varlist'"';

	/* Build big collapse statement: */

	local collapse "collapse (mean)";
	foreach v in `varlist' {;
	    local collapse "`collapse' mean_`v'=`v'";
	};
	local collapse "`collapse' (p50)";
	foreach v in `varlist' {;
	    local collapse "`collapse' p50_`v'=`v'";
	};
	local collapse "`collapse' (sd)";
	foreach v in `varlist' {;
	    local collapse "`collapse' sd_`v'=`v'";
	};
	local collapse "`collapse' (count)";
	foreach v in `varlist' {;
	    local collapse "`collapse' N_`v'=`v'";
	};
	local collapse "`collapse' `if' `in'";
	if "`weight'"!="" local collapse "`collapse' [`weight'`exp']";

	local collapse "`collapse' ,by(_j)";

	preserve;
	sort _j;
	`collapse';

        foreach v in `varlist' {;
	    qui summ mean_`v';
	    local mean_v = r(mean);
	    return scalar mean_`v' = `mean_v';

	    qui summ sd_`v';
	    local sd_v = r(mean);
	    return scalar sd_`v' = `sd_v';

	    qui summ p50_`v';
            local p50_v = r(mean);
	    return scalar p50_`v' = `p50_v';

	    qui summ N_`v';
	    local N_v = r(mean);
	    return scalar N_`v' = `N_v';

            di in text %12s abbrev("`v'",12) " {c |}"
	       as result
	       %9.0g `N_v'  
	       _col(28) %9.0g `mean_v'
	       _col(40) %9.0g `sd_v';
	};

end;
