#delimit;

/* Prints out Latex tables based on summary results stored using
*  summstore.
*
*  _null is a special name used when you want to put some empty columns
*  in the table.
*
*  Version 1.0, Doug McKee, dmckee@ucla.edu, 20jun2008
*/

capture program drop summout;
program summout;
    syntax namelist [using/] [ ,
	   STYLE(string)
	   NOSD(namelist)
	   SHOWMEDIANS(namelist)
	   MEDIANLABEL(string)
	   NLABEL(string)
	   VARLABELS(string asis)
	   FORMAT(string)
	   VARFORMATS(string asis)
	   SAMPLELABELS(string asis)
	   NOHEADER
	   REPLACE
	   ];

    /* Set nosd_<var> macros: */
    foreach nosd in `nosd' {;
        local nosd_`nosd' = 1;
    };

    /* Set showmedian_<var> macros: */
    foreach showmedian in `showmedians' {;
        local showmedian_`showmedian' = 1;
    };

    if `"`medianlabel'"'=="" {;
       if `"`style'"'=="tex" local medianlabel "\hspace{0.1in}(median)";
       else local medianlabel "  (median)";
    };

    if `"`nlabel'"'=="" local nlabel "N";

    /* Set lbl_<var> macros: */
    gettoken v rest : varlabels;
    while `"`v'"'!="" {;
         gettoken label rest : rest;
	 local lbl_`v' = `"`label'"';
	 gettoken v rest : rest;
    };

    /* formats: */
    if `"`format'"'=="" local format "%9.0g";
    gettoken v rest : varformats;
    while `"`v'"'!="" {;
         gettoken fmt rest : rest;
	 local format_`v' = `"`fmt'"';
	 gettoken v rest : rest;
    };

    if `"`style'"'=="" local style "fixed";

    /* Open output file: */

    if `"`using'"'!="" {;
       tempname fh;
       file open `fh' using `using' ,write `replace';
    };

    di;

    /* Construct table header: */
    
    if "`noheader'"=="" {;

        /* Set slbl_name macros: */
    	gettoken name rest : samplelabels;
    	while `"`name'"'!="" {;
            gettoken label rest : rest;
	    local slbl_`name' = `"`label'"';
	    gettoken name rest : rest;
        };
	foreach name in `namelist' {;
	    if `"`slbl_`name''"'=="" local slbl_`name' "`name'";
	};

	if "`style'"=="tex" {;
	    foreach name in `namelist' {;
	    	if "`name'"=="_null" printnullcol "`fh'" "`style'";
		else printcelement `"`fh'"' "`style'" 0 
		     		   `"`slbl_`name''"';
	    };
	    printnewline "`fh'" `style';
	    local cnum 2;
	    foreach name in `namelist' {;
	    	if "`name'"=="_null" {;
		    local cnum = `cnum'+1;
		};
		else {;
		    local cnumend = `cnum'+1;
		    local s `" \cline{`cnum'-`cnumend'}"';
		    printstring "`fh'" `s';
		    local cnum = `cnum'+2;
                };
	    };
	    foreach name in `namelist' {;
	        if "`name'"=="_null" printstring "`fh'" " &";
		else printstring "`fh'" " & Mean & Std Dev";
	    };
	    printhline "`fh'" "`style'";
	    printnewline "`fh'" "`style'";
	};
	else {;
	     local snum 0;
	     foreach name in `namelist' {;
		 if "`name'"=="_null" {;
		     printnullcol "`fh'" `style';
		 };
		 else {;
		     local snum = `snum' + 1;
		     printcelement "`fh'" "`style'" `snum'
		     		   `"`slbl_`name''"';
	         };
	     };
	     printnewline "`fh'" "`style'";
	     local snum 0;
	     foreach name in `namelist' {;
		 if "`name'"=="_null" {;
		     printnullcol "`fh'" `style';
		 };
		 else {;
		     local snum = `snum' + 1;
	             printelement "`fh'" "`style'" `snum' "" 
		 	          "Mean" "Std Dev";
	         };
	     };
	     printnewline "`fh'" "`style'";
	     printhline "`fh'" "`style'" `"`namelist'"';
         };
    };

    /* Construct table body: */

    foreach name in `namelist' {;
        foreach v in ${SS_`name'_varlist} {;
	    if "``v'_done'"=="" {;

	        if `"`format_`v''"'=="" local format_`v' `format';

		/* Row heading: */

		if `"`lbl_`v''"'=="" {;
		   local lbl_`v' : variable label `v';
		};
		if `"`lbl_`v''"'=="" {;
		    local lbl_`v' `v';
		};
		printrowhead "`fh'" `style' `"`lbl_`v''"';

  		/* Statistics: */

		local snum 0;
		foreach col in `namelist' {;
		    if "`col'"=="_null" {;
		        printnullcol "`fh'" `style';
		    };
		    else {;
		        local snum = `snum' + 1;
		        if "`nosd_`v''"=="" local sd ${SS_`col'_sd_`v'};
			else local sd "";
		        printelement "`fh'" `style' `snum' `format_`v''
				     ${SS_`col'_mean_`v'} "`sd'";
		    };
                };
		printnewline "`fh'" `style';

		/* Show median? */

		if "`showmedian_`v''"!="" {;
		    printrowhead "`fh'" `style' `"`medianlabel'"';
		    local snum 0;
		    foreach col in `namelist' {;
		        if "`col'"=="_null" {;
			    printnullcol "`fh'" `style';
		        };
		        else {;
			    local snum = `snum' + 1;
			    printmedian "`fh'" `style' `snum' 
			    		`format_`v''
			    		${SS_`col'_p50_`v'};
			};
		    };
		    printnewline "`fh'" `style';
                };

		local `v'_done 1;
	    };	
	};
    };

    /* Number of observations: */

    printrowhead "`fh'" `style' `"`nlabel'"';
    local snum 0;
    foreach col in `namelist' {;
        if "`col'"=="_null" {;
	   printnullcol "`fh'" `style';
	};
	else {;
	    local snum = `snum' + 1;
       	    local v : word 1 of ${SS_`col'_varlist};
	    printN "`fh'" `style' `snum' ${SS_`col'_N_`v'};
        };
    };
    printnewline "`fh'" `style';

    if `"`using'"'!="" file close `fh';

end;


capture program drop printstring;
program printstring;
	args fh s;

        if `"`fh'"'!="" file write `fh' `"`s'"';
        di `"`s'"' _continue;
end;

capture program drop printrowhead;
program printrowhead;
	args fh style rowhead;

	printstring "`fh'" `"`rowhead'"';
end;

capture program drop printelement;
program printelement;
	args fh style snum fmt part1 part2;

	if `"`fmt'"'!="" local fpart1 : display `fmt' `part1';
	else local fpart1 `"`part1'"';

	if `"`part2'"'!="" {;
	    if `"`fmt'"'!="" local fpart2 : display `fmt' `part2';
	    else local fpart2 `"`part2'"';
	};

	local pad1 = 11-length(`"`fpart1'"');
	local pad2 = 11-length(`"`fpart2'"');
	local c1 = 18 + (`snum'-1)*22 + `pad1';
	local c2 = 18 + (`snum'-1)*22 + 11 + `pad2';

	if `"`fh'"'!="" {;

	    if "`style'"=="tex" {;
	        file write `fh' `" & `fpart1' & `fpart2'"';
	    };
	    if "`style'"=="tab" {;
	        file write `fh' _tab `"`fpart1'"' _tab `"`fpart2'"';
	    };
	    if "`style'"=="fixed" {;
	        file write `fh' _col(`c1') `"`fpart1'"' 
		     	   	_col(`c2') `"`fpart2'"';
	    };
	};

	if "`style'"=="tex" {;
	   di " & `fpart1' & `fpart2'" _continue;
	};
	else {;
	   di _col(`c1') `"`fpart1'"' _col(`c2') `"`fpart2'"' _continue;
	};
end;

/* Print a centered element */
capture program drop printcelement;
program printcelement;
	args fh style snum s;

	if "`style'"=="tex" {;
	   printstring `"`fh'"' `" & \multicolumn{2}{c}{`s'}"';
        };
	else {;
	    local len = length(`"`s'"');
	    local c1 = 18 + (`snum'-1)*22 + floor((22-`len')/2);
	    di _col(`c1') `"`s'"' _continue;
	    if "`fh'"!="" {;
	        if "`style'"=="tab" {;
	            file write `fh' _tab `"`s'"' _tab;
	        };
	        if "`style'"=="fixed" {;
	            file write `fh' _col(`c1') `"`s'"';
	        };
	    };
	};
end;

capture program drop printmedian;
program printmedian;
	args fh style snum fmt median;

	local val : display `fmt' `median';
   	local val = trim("`val'");
	local val "(`val')";
	printelement "`fh'" `style' `snum' "" `"`val'"' "";

end;

capture program drop printN;
program printN;
	args fh style snum N;

	if "`style'"=="tex" {;

	    if `"`fh'"'!="" file write `fh' 
	       		    	 " & \multicolumn{2}{c}{`N'}";
	    di " & \multicolumn{2}{c}{`N'}" _continue;
	};
	else {;
	   printelement "`fh'" `style' `snum' %9.0g "`N'" "";
	};
end;

capture program drop printnullcol;
program printnullcol;
	args fh style;

	if "`style'"=="tex" {;
	   printstring "`fh'" " & ";
	};
	if "`style'"=="tab" {;
	   if `"`fh'"'!="" file write `fh' _tab;
	};
end;

capture program drop printnewline;
program printnewline;
	args fh style;

	if "`style'"=="tex" {;
	    local end "\\";
	};
   	if `"`fh'"'!="" file write `fh' `"`end'"' _newline;
	di `"`end'"';
end;


/* printhline:

   Style=tex: \hline is written to screen and file.
   Style=fixed: SCML hline is written to screen and dashed line is
   		written to file.
   Style=tab: SCML hline is written to screen and nothing is 
   	      written to file. 
*/

capture program drop printhline;
program printhline;
	args fh style namelist;

	if "`style'"=="tex" {;
	   printstring "`fh'" " \hline";
	};
	else {;
	    local n 18;
	    foreach name in `namelist' {;
	        if `"`name'"'!="_null" local n = `n' + 22;
	    };
	    di "{hline `n'}";
	    if "`style'"=="fixed" & "`fh'"!="" {;
	       file write `fh' _dup(`n') "-" _newline;    
	    };
	};
end;
